<!doctype html>
<html>

<head>
  <!-- We prefer static data than CDN because client may not have internet connection -->
  <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/tether.min.css') }}" />
  <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/bootstrap.min.css') }}" />
  <link rel="stylesheet" type="text/css" href="{{ url_for('static', filename='css/main.css') }}" />
  <title>chainerboard: {% block title %}{% endblock %}</title>
  {% block header %}{% endblock %}
</head>

<body>

<nav class="navbar navbar-default navbar-fixed-top">
  <div class="navbar-header">
    <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#bs-example-navbar-collapse-1" aria-expanded="false">
      <span class="sr-only">Toggle navigation</span>
      <span class="icon-bar"></span>
      <span class="icon-bar"></span>
      <span class="icon-bar"></span>
    </button>
    <a class="navbar-brand" href="{{url_for('index')}}">chainerboard</a>
  </div>
  <div class="collapse navbar-collapse" id="bs-example-navbar-collapse-1">
    <ul class="nav navbar-nav">
      <li class="{% block nav_active_events %}{% endblock %}">
        <a href="{{url_for('events')}}"><span>Events </span></a>
      </li>
      <li class="{% block nav_active_histograms %}{% endblock %}">
        <a href="{{url_for('histograms')}}"><span>Histograms </span></a>
      </li>
    </ul>
  </div>
</nav>

<div class="container-fluid">
{% block content %}{% endblock %}
</div><!-- /.container-fluid -->

</body>


<footer>
  <script src="{{ url_for('static', filename='js/jquery-2.1.4.min.js') }}"></script>
  <script src="{{ url_for('static', filename='js/tether.min.js') }}"></script>
  <script src="{{ url_for('static', filename='js/bootstrap.min.js') }}"></script>
  <script src="{{ url_for('static', filename='js/angular.min.js') }}"></script>
  <script src="{{ url_for('static', filename='js/plotly-1.28.3.min.js') }}"></script>
  <script type="text/javascript">
    var $SCRIPT_ROOT = {{ request.script_root|tojson|safe }};
  </script>
  <script src="{{ url_for('static', filename='js/common.js') }}"></script>
  {% block footer %}{% endblock %}
</footer>

</html>


